package org.processmining.humbertogarciacaballero.algorithms;

import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.processmining.framework.plugin.PluginContext;
import org.processmining.models.graphbased.AttributeMap;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.plugins.petrinet.behavioralanalysis.woflan.Woflan;
import org.processmining.plugins.petrinet.behavioralanalysis.woflan.WoflanDiagnosis;

import com.google.common.collect.HashMultiset;
import com.google.common.collect.Multiset;

public class ProcessMiningAlgorithms {

	/**
	 * The method that implements your algorithm.
	 * 
	 * Note that this method only uses the boolean which is stored in the
	 * parameters. Nevertheless, it could have used the integer and/or the
	 * String as well.
	 * 
	 * @param context
	 *            The context where to run in.
	 * @param input1
	 *            The first input.
	 * @param input2
	 *            The second input.
	 * @param parameters
	 *            The parameters to use.
	 * @return The output.
	 */
	public static void calculateParallelism(PluginContext context, PetrinetGraph pnet) {

	}

//	public static void calculateParallelism(PluginContext context, AcceptingPetriNet pnet) {
//
//	}

	public static Set<Transition> calculateChoices(PluginContext context, PetrinetGraph net) {
		Set<Transition> globalTranPath = new HashSet<>();
		findChoices(calculateStartingMarking(net), calculateEndingMarking(net), net, new HashSet<Transition>(),
				globalTranPath);
		return globalTranPath;
	}

//	public static void calculateChoices(PluginContext context, AcceptingPetriNet net) {
//
//	}
	
	// Auxiliar methods
	private static void findChoices(Marking currMarking, Marking pEndSet, PetrinetGraph pnet,
			Set<Transition> currTranPath, Set<Transition> globalTranPath) {
		Set<Transition> enabledTransitions = new HashSet<Transition>();
		enabledTransitions = getEnabledTransitions(pnet, currMarking);

		if (pEndSet.containsAll(currMarking)) {
			globalTranPath.addAll(currTranPath);
		} else {
			for (Transition enabledTransition : enabledTransitions) {
				if (currTranPath.contains(enabledTransition)) //if looped back, return
					return;
				Set<Transition> currTranPathNew = new HashSet<Transition>(currTranPath);
				currTranPathNew.add(enabledTransition);
				Marking newMarking = new Marking(currMarking);
				for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> edge : pnet
						.getInEdges(enabledTransition))
					newMarking.remove(edge.getSource());
				for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> edge : pnet
						.getOutEdges(enabledTransition))
					newMarking.add((Place) edge.getTarget());
				findChoices(newMarking, pEndSet, pnet, currTranPathNew, globalTranPath);
			}
		}
		return;
	}

	private static Set<Transition> getEnabledTransitions(PetrinetGraph ptnet, Marking marking) {
		return getEnabledTransitions(ptnet, HashMultiset.create(marking));
	}

	private static Set<Transition> getEnabledTransitions(PetrinetGraph ptnet, Multiset<Place> marking) {
		Set<Transition> enabledTransitions = new HashSet<Transition>();
		Collection<Transition> netTransitions = ptnet.getTransitions();
		for (Transition t : netTransitions) {
			Collection<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> inflow = ptnet.getInEdges(t);
			boolean enabled = true;
			for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> edge : inflow) {
				if (!marking.contains((edge.getSource()))) {
					enabled = false;
					break;
				}
			}
			if (enabled)
				enabledTransitions.add(t);
		}
		return enabledTransitions;
	}

	private static Marking calculateStartingMarking(PetrinetGraph net) {
		Marking marking = new Marking();

		for (PetrinetNode node : net.getNodes()) {
			if (node instanceof Place && net.getInEdges(node).isEmpty()) {
				marking.add((Place) node);
			}
		}

		return marking;
	}

	private static Marking calculateEndingMarking(PetrinetGraph net) {
		Marking marking = new Marking();
		
		for (PetrinetNode node : net.getNodes()) {
			if (node instanceof Place && net.getOutEdges(node).isEmpty()) {
				marking.add((Place) node);
			}
		}

		return marking;
	}
	
	public static WoflanDiagnosis diagnosePetrinet(PluginContext context, Petrinet net) {
		Woflan w = new Woflan();
		try {
			WoflanDiagnosis diagnosis = w.diagnose(context, net);
			return diagnosis;
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		return null;
	}
	
	public static Map<String, String> preprocessDiagnosisPetrinet(PluginContext context, Petrinet net) {
		Map<String, String> labels = new HashMap<>();
		
		int label = 0;
		for (PetrinetNode node : net.getNodes()) {
			labels.put(node.getId().toString(), node.getLabel());
			node.getAttributeMap().put(AttributeMap.LABEL, String.valueOf(label++));
		}
		
		return labels;
	}
	
	public static void postprocessDiagnosisPetrinet(PluginContext context, Petrinet net, Map<String, String> labels) {
		for (PetrinetNode node : net.getNodes()) {
			node.getAttributeMap().put(AttributeMap.LABEL, String.valueOf(labels.get(node.getId().toString())));
		}
	}
}
