package org.processmining.humbertogarciacaballero.models.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;

import javax.swing.AbstractButton;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JColorChooser;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JTabbedPane;
import javax.swing.KeyStroke;
import javax.swing.SpinnerNumberModel;
import javax.swing.border.EmptyBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.processmining.humbertogarciacaballero.models.controller.PNetVisualizerController;
import org.processmining.humbertogarciacaballero.models.model.PNConstants;
import org.processmining.humbertogarciacaballero.models.model.layout.data.LayoutOrientation;
import org.processmining.humbertogarciacaballero.models.model.layout.data.LineType;
import org.processmining.humbertogarciacaballero.models.model.layout.stage.emplacer.EmplacingStrategy;
import org.processmining.humbertogarciacaballero.models.model.layout.stage.layerer.LayeringStrategy;
import org.processmining.humbertogarciacaballero.models.model.layout.stage.sequencer.SequencingStrategy;

import com.jgoodies.forms.layout.ColumnSpec;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.forms.layout.FormSpecs;
import com.jgoodies.forms.layout.RowSpec;

import prefuse.util.ColorLib;

public class LayoutParametersGUI extends JDialog {

	private static final long serialVersionUID = 1L;

	// Look
	private final Font titleFont = new Font("Arial", Font.BOLD, 13);

	// Components
	// Layout options
	private final JPanel contentPanel = new JPanel();
	private final JCheckBox chckbxEnabledFirstStages;
	private final JComboBox<LayeringStrategy> cbLayeringStrategy;
	private final JComboBox<LayoutOrientation> cbLayoutOrientation;
	private final JCheckBox chckbxEnableSimplifier;
	private final JCheckBox chckbxSimplifyAll;
	private final JCheckBox chckbxSimplifyLongEdges;
	private final JSpinner spCrossingThreshold;
	private final JSpinner spTotalCrossings;
	private final JSpinner spMaxLoops;
	private final JCheckBox chckbxSimplifySameLayer;
	private final JCheckBox chckbxReorderNodes;
	private final JSpinner spIntracellSpacing;
	private final JSpinner spLayerSpacing;
	private final JSpinner spEdgeLandingOffset;
	private final JComboBox<SequencingStrategy> cbSequencingStrategy;
	private final JComboBox<EmplacingStrategy> cbEmplacingStrategy;
	private final JComboBox<LineType> cbLineType;
	private final JCheckBox chckbxEdgeDirection;
	private final JCheckBox chckbxBarDrawing;

	// Other options
	private final JSpinner spTransitionsWidth;
	private final JSpinner spTransitionsHeight;
	private final JCheckBox chckbxTransitionsInOutDegree;
	private final JSpinner spPlacesWidth;
	//    private final JSpinner spPlacesHeight;
	private final JSpinner spFontSize;
	private final JSpinner spDegreeFontSize;
	private final JSpinner spAnimationLength;

	// Colors
	private Color edgeColor;
	private Color strokeColor;

	// Controller
	private final PNetVisualizerController controller = PNetVisualizerController.getInstance();

	/**
	 * Create the dialog.
	 */
	public LayoutParametersGUI(JFrame root) {
		super(root);
		addEscapeListener(this);
		setMinimumSize(new Dimension(400, 700));
		setDefaultCloseOperation(HIDE_ON_CLOSE);
		setTitle("Petri net viewer");
		setBounds(100, 100, 293, 700);
		setLocationRelativeTo(null);
		getContentPane().setLayout(new BorderLayout());
		contentPanel.setBorder(new EmptyBorder(5, 5, 5, 5));
		getContentPane().add(contentPanel, BorderLayout.CENTER);
		contentPanel.setLayout(new BorderLayout(0, 0));
		setAlwaysOnTop(true);
		{
			JTabbedPane tabbedPane = new JTabbedPane(JTabbedPane.TOP);
			contentPanel.add(tabbedPane, BorderLayout.CENTER);
			{
				JScrollPane panel = new JScrollPane();
				tabbedPane.addTab("Layout options", null, panel, null);
				{
					JPanel panel_1 = new JPanel();
					panel.setViewportView(panel_1);
					panel_1.setLayout(new BorderLayout(0, 0));
					{
						JPanel panel_2_1 = new JPanel();
						panel_1.add(panel_2_1, BorderLayout.NORTH);
						panel_2_1.setLayout(new FormLayout(
								new ColumnSpec[] { FormSpecs.RELATED_GAP_COLSPEC, FormSpecs.DEFAULT_COLSPEC,
										FormSpecs.RELATED_GAP_COLSPEC, ColumnSpec.decode("default:grow"),
										FormSpecs.RELATED_GAP_COLSPEC, },
								new RowSpec[] { FormSpecs.RELATED_GAP_ROWSPEC, FormSpecs.DEFAULT_ROWSPEC,
										FormSpecs.RELATED_GAP_ROWSPEC, FormSpecs.DEFAULT_ROWSPEC,
										FormSpecs.RELATED_GAP_ROWSPEC, FormSpecs.DEFAULT_ROWSPEC,
										FormSpecs.RELATED_GAP_ROWSPEC, FormSpecs.DEFAULT_ROWSPEC,
										FormSpecs.RELATED_GAP_ROWSPEC, FormSpecs.DEFAULT_ROWSPEC,
										FormSpecs.RELATED_GAP_ROWSPEC, FormSpecs.RELATED_GAP_ROWSPEC,
										FormSpecs.DEFAULT_ROWSPEC, FormSpecs.RELATED_GAP_ROWSPEC,
										FormSpecs.DEFAULT_ROWSPEC, FormSpecs.RELATED_GAP_ROWSPEC,
										RowSpec.decode("default:grow"), FormSpecs.RELATED_GAP_ROWSPEC,
										FormSpecs.DEFAULT_ROWSPEC, FormSpecs.RELATED_GAP_ROWSPEC,
										FormSpecs.DEFAULT_ROWSPEC, FormSpecs.RELATED_GAP_ROWSPEC,
										FormSpecs.DEFAULT_ROWSPEC, FormSpecs.RELATED_GAP_ROWSPEC,
										FormSpecs.DEFAULT_ROWSPEC, FormSpecs.RELATED_GAP_ROWSPEC,
										FormSpecs.DEFAULT_ROWSPEC, FormSpecs.RELATED_GAP_ROWSPEC,
										FormSpecs.DEFAULT_ROWSPEC, FormSpecs.RELATED_GAP_ROWSPEC,
										FormSpecs.DEFAULT_ROWSPEC, FormSpecs.RELATED_GAP_ROWSPEC,
										FormSpecs.DEFAULT_ROWSPEC, FormSpecs.RELATED_GAP_ROWSPEC,
										FormSpecs.DEFAULT_ROWSPEC, FormSpecs.RELATED_GAP_ROWSPEC,
										FormSpecs.DEFAULT_ROWSPEC, FormSpecs.RELATED_GAP_ROWSPEC,
										FormSpecs.DEFAULT_ROWSPEC, FormSpecs.RELATED_GAP_ROWSPEC,
										FormSpecs.DEFAULT_ROWSPEC, FormSpecs.RELATED_GAP_ROWSPEC,
										FormSpecs.DEFAULT_ROWSPEC, }));
						{
							JLabel lblCommonLayoutParameters = new JLabel("Common layout parameters");
							lblCommonLayoutParameters.setFont(titleFont);
							panel_2_1.add(lblCommonLayoutParameters, "2, 2, 3, 1, center, default");
						}
						{
							JLabel lblEnabled = new JLabel("Enabled:");
							panel_2_1.add(lblEnabled, "2, 4, right, default");
						}
						{
							chckbxEnabledFirstStages = new JCheckBox("");
							chckbxEnabledFirstStages.addChangeListener(new ChangeListener() {
								@Override
								public void stateChanged(ChangeEvent e) {
									JCheckBox cb = ((JCheckBox) e.getSource());
									boolean enabled = cb.isSelected();
									setFirstStagesEnabled(enabled);
								}
							});
							panel_2_1.add(chckbxEnabledFirstStages, "4, 4");
						}
						{
							JLabel lblLayeringStrategy = new JLabel("Layering strategy:");
							panel_2_1.add(lblLayeringStrategy, "2, 6, right, default");
						}
						{
							cbLayeringStrategy = new JComboBox<LayeringStrategy>();
							cbLayeringStrategy
									.setModel(new DefaultComboBoxModel<LayeringStrategy>(LayeringStrategy.values()));
							panel_2_1.add(cbLayeringStrategy, "4, 6, fill, default");
						}
						{
							JLabel lblSequencer = new JLabel("Sequencing strategy:");
							panel_2_1.add(lblSequencer, "2, 8, right, default");
						}
						{
							cbSequencingStrategy = new JComboBox<SequencingStrategy>();
							cbSequencingStrategy.setModel(
									new DefaultComboBoxModel<SequencingStrategy>(SequencingStrategy.values()));
							panel_2_1.add(cbSequencingStrategy, "4, 8, fill, default");
						}
						{
							JLabel lblSimplifierParameters = new JLabel("Simplifier parameters");
							lblSimplifierParameters.setFont(titleFont);
							panel_2_1.add(lblSimplifierParameters, "2, 13, 3, 1, center, default");
						}
						{
							JLabel lblEnable = new JLabel("Enabled:");
							panel_2_1.add(lblEnable, "2, 15, right, default");
						}
						{
							chckbxEnableSimplifier = new JCheckBox("");
							chckbxEnableSimplifier.addActionListener(new ActionListener() {
								@Override
								public void actionPerformed(ActionEvent e) {
									AbstractButton abstractButton = (AbstractButton) e.getSource();
									boolean selected = abstractButton.getModel().isSelected();
									setSimplifierPropertiesEnabled(selected);
								}
							});
							panel_2_1.add(chckbxEnableSimplifier, "4, 15");
						}
						{
							JLabel lblSimplifyShortEdges = new JLabel("Short edges:");
							panel_2_1.add(lblSimplifyShortEdges, "2, 17, right, default");
						}
						{
							JPanel panel_2 = new JPanel();
							FlowLayout flowLayout = (FlowLayout) panel_2.getLayout();
							flowLayout.setHgap(0);
							flowLayout.setVgap(0);
							flowLayout.setAlignment(FlowLayout.LEFT);
							chckbxSimplifyAll = new JCheckBox("");
							chckbxSimplifyAll.addActionListener(new ActionListener() {
								@Override
								public void actionPerformed(ActionEvent e) {
									AbstractButton abstractButton = (AbstractButton) e.getSource();
									boolean selected = abstractButton.getModel().isSelected();
									setSimplifyShortLongPropertiesEnabled(selected || chckbxSimplifyLongEdges.isSelected());
								}
							});
							panel_2.add(chckbxSimplifyAll);
							panel_2_1.add(panel_2, "4, 17, fill, fill");
							{
								Component horizontalStrut = Box.createHorizontalStrut(20);
								panel_2.add(horizontalStrut);
							}
							{
								JLabel lblLongEdges = new JLabel("Long edges:");
								panel_2.add(lblLongEdges);
							}
							{
								chckbxSimplifyLongEdges = new JCheckBox("");
								chckbxSimplifyLongEdges.addActionListener(new ActionListener() {
									@Override
									public void actionPerformed(ActionEvent e) {
										AbstractButton abstractButton = (AbstractButton) e.getSource();
										boolean selected = abstractButton.getModel().isSelected();
										setSimplifyShortLongPropertiesEnabled(
												selected || chckbxSimplifyLongEdges.isSelected());
									}
								});
								panel_2.add(chckbxSimplifyLongEdges);
							}
						}
						{
							JLabel lblNewLabel = new JLabel("Edge direction:");
							panel_2_1.add(lblNewLabel, "2, 19, right, default");
						}
						{
							JPanel panel_3 = new JPanel();
							FlowLayout flowLayout = (FlowLayout) panel_3.getLayout();
							flowLayout.setHgap(0);
							flowLayout.setVgap(0);
							flowLayout.setAlignment(FlowLayout.LEFT);
							chckbxEdgeDirection = new JCheckBox("");
							panel_3.add(chckbxEdgeDirection);
							panel_2_1.add(panel_3, "4, 19, fill, fill");
							{
								Component horizontalStrut = Box.createHorizontalStrut(20);
								panel_3.add(horizontalStrut);
							}
							{
								JLabel lblDrawBars = new JLabel("Draw bars:");
								panel_3.add(lblDrawBars);
							}
							{
								chckbxBarDrawing = new JCheckBox("");
								panel_3.add(chckbxBarDrawing);
							}
						}
						{
							JLabel lblNewLabel = new JLabel("Reorder nodes:");
							panel_2_1.add(lblNewLabel, "2, 21, right, default");
						}
						{
							JPanel panel_4 = new JPanel();
							FlowLayout flowLayout = (FlowLayout) panel_4.getLayout();
							flowLayout.setHgap(0);
							flowLayout.setVgap(0);
							flowLayout.setAlignment(FlowLayout.LEFT);
							chckbxReorderNodes = new JCheckBox("");
							panel_4.add(chckbxReorderNodes);
							panel_2_1.add(panel_4, "4, 21, fill, fill");
							{
								Component horizontalStrut = Box.createHorizontalStrut(20);
								panel_4.add(horizontalStrut);
							}
							{
								JLabel lblSimplifySameLayer = new JLabel("Simplify same layer:");
								panel_4.add(lblSimplifySameLayer);
							}
							{
								chckbxSimplifySameLayer = new JCheckBox("");
								panel_4.add(chckbxSimplifySameLayer);
							}
						}
						{
							JLabel lblCrossingThreshold = new JLabel("Crossing threshold:");
							panel_2_1.add(lblCrossingThreshold, "2, 23, right, default");
						}
						{
							spCrossingThreshold = new JSpinner();
							spCrossingThreshold.setModel(
									new SpinnerNumberModel(new Integer(5), new Integer(0), null, new Integer(1)));
							panel_2_1.add(spCrossingThreshold, "4, 23");
						}
						{
							JLabel lblTotalThreshold = new JLabel("Total threshold:");
							panel_2_1.add(lblTotalThreshold, "2, 25, right, default");
						}
						{
							spTotalCrossings = new JSpinner();
							spTotalCrossings.setModel(
									new SpinnerNumberModel(new Integer(15), new Integer(0), null, new Integer(1)));
							panel_2_1.add(spTotalCrossings, "4, 25");
						}
						{
							JLabel lblMaxLoops = new JLabel("Max. loops:");
							panel_2_1.add(lblMaxLoops, "2, 27, right, default");
						}
						{
							spMaxLoops = new JSpinner();
							spMaxLoops.setModel(
									new SpinnerNumberModel(new Integer(24), new Integer(0), null, new Integer(1)));
							panel_2_1.add(spMaxLoops, "4, 27");
						}
						{
							JLabel lblEmplacerParameters = new JLabel("Emplacer parameters");
							lblEmplacerParameters.setFont(titleFont);
							panel_2_1.add(lblEmplacerParameters, "2, 31, 3, 1, center, default");
						}
						{
							JLabel lblOrientation = new JLabel("Orientation:");
							panel_2_1.add(lblOrientation, "2, 33, right, default");
						}
						{
							cbLayoutOrientation = new JComboBox<LayoutOrientation>();
							cbLayoutOrientation
									.setModel(new DefaultComboBoxModel<LayoutOrientation>(LayoutOrientation.values()));
							panel_2_1.add(cbLayoutOrientation, "4, 33, fill, default");
						}
						{
							JLabel lblAlgorithm = new JLabel("Strategy:");
							panel_2_1.add(lblAlgorithm, "2, 35, right, default");
						}
						{
							cbEmplacingStrategy = new JComboBox<EmplacingStrategy>();
							cbEmplacingStrategy
									.setModel(new DefaultComboBoxModel<EmplacingStrategy>(EmplacingStrategy.values()));
							panel_2_1.add(cbEmplacingStrategy, "4, 35, fill, default");
						}
						{
							JLabel lblIntracellSpacing = new JLabel("Intracell spacing:");
							panel_2_1.add(lblIntracellSpacing, "2, 37, right, default");
						}
						{
							spIntracellSpacing = new JSpinner();
							spIntracellSpacing.setModel(
									new SpinnerNumberModel(new Integer(1), new Integer(1), null, new Integer(1)));
							panel_2_1.add(spIntracellSpacing, "4, 37");
						}
						{
							JLabel lblLayerSpacing = new JLabel("Layer spacing:");
							panel_2_1.add(lblLayerSpacing, "2, 39, right, default");
						}
						{
							spLayerSpacing = new JSpinner();
							spLayerSpacing.setModel(
									new SpinnerNumberModel(new Integer(1), new Integer(1), null, new Integer(1)));
							panel_2_1.add(spLayerSpacing, "4, 39");
						}
						{
							JLabel lblEdgeLandingOffset = new JLabel("Edge landing offset:");
							{
								JLabel lblLineType = new JLabel("Line type:");
								panel_2_1.add(lblLineType, "2, 41, right, default");
							}
							{
								cbLineType = new JComboBox<LineType>();
								cbLineType.setModel(new DefaultComboBoxModel<LineType>(LineType.values()));
								panel_2_1.add(cbLineType, "4, 41");
							}
							panel_2_1.add(lblEdgeLandingOffset, "2, 43, right, default");
						}
						{
							spEdgeLandingOffset = new JSpinner();
							spEdgeLandingOffset.setModel(
									new SpinnerNumberModel(new Float(10), new Float(0), null, new Float(0.5)));
							panel_2_1.add(spEdgeLandingOffset, "4, 43");
						}
					}
				}
			}
			{
				JScrollPane panel = new JScrollPane();
				tabbedPane.addTab("Other options", null, panel, null);
				{
					JPanel panel_1 = new JPanel();
					panel.setViewportView(panel_1);
					panel_1.setLayout(new BoxLayout(panel_1, BoxLayout.X_AXIS));
					{
						JPanel panel_2_1 = new JPanel();
						panel_1.add(panel_2_1);
						panel_2_1.setLayout(new FormLayout(
								new ColumnSpec[] { FormSpecs.RELATED_GAP_COLSPEC, FormSpecs.DEFAULT_COLSPEC,
										FormSpecs.RELATED_GAP_COLSPEC, ColumnSpec.decode("default:grow"),
										FormSpecs.RELATED_GAP_COLSPEC, },
								new RowSpec[] { FormSpecs.RELATED_GAP_ROWSPEC, FormSpecs.DEFAULT_ROWSPEC,
										FormSpecs.RELATED_GAP_ROWSPEC, FormSpecs.DEFAULT_ROWSPEC,
										FormSpecs.RELATED_GAP_ROWSPEC, FormSpecs.DEFAULT_ROWSPEC,
										FormSpecs.RELATED_GAP_ROWSPEC, FormSpecs.DEFAULT_ROWSPEC,
										FormSpecs.RELATED_GAP_ROWSPEC, FormSpecs.DEFAULT_ROWSPEC,
										FormSpecs.RELATED_GAP_ROWSPEC, FormSpecs.DEFAULT_ROWSPEC,
										FormSpecs.RELATED_GAP_ROWSPEC, FormSpecs.DEFAULT_ROWSPEC,
										FormSpecs.RELATED_GAP_ROWSPEC, FormSpecs.DEFAULT_ROWSPEC,
										FormSpecs.RELATED_GAP_ROWSPEC, FormSpecs.DEFAULT_ROWSPEC, }));
						{
							JLabel lblNewLabel_1 = new JLabel("Transitions width:");
							panel_2_1.add(lblNewLabel_1, "2, 2, right, default");
						}
						{
							spTransitionsWidth = new JSpinner();
							spTransitionsWidth
									.setModel(new SpinnerNumberModel(new Integer(PNConstants.TRANSITIONS_DEFAULT_WIDTH),
											new Integer(1), null, new Integer(1)));
							panel_2_1.add(spTransitionsWidth, "4, 2, fill, default");
						}
						{
							JLabel lblTransitionsHeight = new JLabel("Transitions height:");
							panel_2_1.add(lblTransitionsHeight, "2, 4, right, default");
						}
						{
							spTransitionsHeight = new JSpinner();
							spTransitionsHeight.setModel(
									new SpinnerNumberModel(new Integer(PNConstants.TRANSITIONS_DEFAULT_HEIGHT),
											new Integer(1), null, new Integer(1)));
							panel_2_1.add(spTransitionsHeight, "4, 4, fill, default");
						}
						{
							JLabel lblTransitionsInoutDegree = new JLabel("Transitions in/out degree:");
							panel_2_1.add(lblTransitionsInoutDegree, "2, 6, right, default");
						}
						{
							chckbxTransitionsInOutDegree = new JCheckBox("");
							panel_2_1.add(chckbxTransitionsInOutDegree, "4, 6");
						}
						{
							JLabel lblNewLabel_2 = new JLabel("Places size:");
							panel_2_1.add(lblNewLabel_2, "2, 8, right, default");
						}
						{
							spPlacesWidth = new JSpinner();
							spPlacesWidth.setModel(new SpinnerNumberModel(new Integer(PNConstants.PLACES_DEFAULT_SIZE),
									new Integer(1), null, new Integer(1)));
							panel_2_1.add(spPlacesWidth, "4, 8, fill, default");
						}
						//                        {
						//                            JLabel lblPlacesHeight = new JLabel("Places height:");
						//                            panel_2_1.add(lblPlacesHeight, "2, 8, right, default");
						//                        }
						//                        {
						//                            spPlacesHeight = new JSpinner();
						//                            spPlacesHeight
						//                                    .setModel(new SpinnerNumberModel(new Integer(PNConstants.PLACES_DEFAULT_HEIGHT),
						//                                            new Integer(1), null, new Integer(1)));
						//                            panel_2_1.add(spPlacesHeight, "4, 8, fill, default");
						//                        }
						{
							JLabel lblNewLabel_3 = new JLabel("Label size:");
							panel_2_1.add(lblNewLabel_3, "2, 10, right, default");
						}
						{
							spFontSize = new JSpinner();
							spFontSize.setModel(new SpinnerNumberModel(new Integer(PNConstants.FONT_DEFAULT_SIZE),
									new Integer(0), null, new Integer(1)));
							panel_2_1.add(spFontSize, "4, 10, fill, default");
						}
						{
							JLabel lblNewLabel_3 = new JLabel("Degree label size:");
							panel_2_1.add(lblNewLabel_3, "2, 12, right, default");
						}
						{
							spDegreeFontSize = new JSpinner();
							spDegreeFontSize
									.setModel(new SpinnerNumberModel(new Integer(PNConstants.FONT_DEGREE_DEFAULT_SIZE),
											new Integer(0), null, new Integer(1)));
							panel_2_1.add(spDegreeFontSize, "4, 12, fill, default");
						}
						{
							JLabel lblAnimationLast = new JLabel("Animation length:");
							panel_2_1.add(lblAnimationLast, "2, 14, right, default");
						}
						{
							spAnimationLength = new JSpinner();
							spAnimationLength.setModel(new SpinnerNumberModel(
									new Long(PNConstants.ANIMATION_LENGTH_DEFAULT), new Long(0), null, new Long(100)));
							panel_2_1.add(spAnimationLength, "4, 14, fill, default");
						}
						{
							JLabel lblEdgesColor = new JLabel("Edges' color:");
							panel_2_1.add(lblEdgesColor, "2, 16, right, default");
						}
						{
							JButton btnChoose = new JButton("Choose");
							btnChoose.addActionListener(new ActionListener() {
								public void actionPerformed(ActionEvent e) {
									edgeColor = JColorChooser.showDialog(LayoutParametersGUI.this, "Pick a color",
											edgeColor);
								}
							});
							panel_2_1.add(btnChoose, "4, 16");
						}
						{
							JLabel lblNodesborderColor = new JLabel("Nodes'border color:");
							panel_2_1.add(lblNodesborderColor, "2, 18, right, default");
						}
						{
							JButton btnChoose_1 = new JButton("Choose");
							btnChoose_1.addActionListener(new ActionListener() {
								public void actionPerformed(ActionEvent e) {
									strokeColor = JColorChooser.showDialog(LayoutParametersGUI.this, "Pick a color",
											strokeColor);
								}
							});
							panel_2_1.add(btnChoose_1, "4, 18");
						}
					}
				}
			}
		}
		{
			JPanel buttonPane = new JPanel();
			buttonPane.setLayout(new FlowLayout(FlowLayout.RIGHT));
			getContentPane().add(buttonPane, BorderLayout.SOUTH);
			{
				JButton btnSaveApply = new JButton("Apply");
				btnSaveApply.addActionListener(new ActionListener() {
					@Override
					public void actionPerformed(ActionEvent e) {
						saveChanges();
					}
				});
				buttonPane.add(btnSaveApply);
			}
			{
				JButton okButton = new JButton("Apply & Save");
				okButton.setActionCommand("OK");
				okButton.addActionListener(new ActionListener() {
					@Override
					public void actionPerformed(ActionEvent e) {
						saveChanges();
						setVisible(false);
					}
				});
				buttonPane.add(okButton);
				getRootPane().setDefaultButton(okButton);
			}
			{
				JButton cancelButton = new JButton("Close");
				cancelButton.setActionCommand("Cancel");
				cancelButton.addActionListener(new ActionListener() {
					@Override
					public void actionPerformed(ActionEvent e) {
						setVisible(false);
					}
				});
				buttonPane.add(cancelButton);
			}
		}

		loadFormValues();
	}

	public void loadFormValues() {
		// Layout options
		chckbxEnabledFirstStages.setSelected(controller.isFirstStagesEnabled());
		cbLayeringStrategy.setSelectedItem(controller.getLayeringStrategy());
		cbSequencingStrategy.setSelectedItem(controller.getSequencingStrategy());
		setFirstStagesEnabled(controller.isFirstStagesEnabled());
		chckbxEnableSimplifier.setSelected(controller.isSimplifierEnabled());
		chckbxSimplifyAll.setSelected(controller.isSimplifyShortEdgesEnabled());
		chckbxSimplifyLongEdges.setSelected(controller.isSimplifyLongEdgesEnabled());
		chckbxEdgeDirection.setSelected(controller.isEdgeDirectionEnabled());
		chckbxBarDrawing.setSelected(controller.isBarDrawingEnabled());
		spCrossingThreshold.setValue(controller.getSimplifierCrossingThreshold());
		spTotalCrossings.setValue(controller.getSimplifierTotalCrossingThreshold());
		spMaxLoops.setValue(controller.getSimplifierMaxLoops());
		chckbxSimplifySameLayer.setSelected(controller.isSimplifySameLayer());
		chckbxReorderNodes.setSelected(controller.isReorderNodesEnabled());
		setSimplifierPropertiesEnabled(controller.isSimplifierEnabled());
		spIntracellSpacing.setValue(controller.getIntracellSpacing());
		spLayerSpacing.setValue(controller.getLayerSpacing());
		spEdgeLandingOffset.setValue(controller.getEdgeLandingOffset());
		cbLayoutOrientation.setSelectedItem(controller.getLayoutOrientation());
		cbEmplacingStrategy.setSelectedItem(controller.getEmplacingStrategy());
		cbLineType.setSelectedItem(controller.getLineType());

		// Other options
		spTransitionsWidth.setValue(controller.getTransitionsWidth());
		spTransitionsHeight.setValue(controller.getTransitionsHeight());
		chckbxTransitionsInOutDegree.setSelected(controller.isTransitionsInOutDegreeEnabled());
		spPlacesWidth.setValue(controller.getPlacesSize());
		spFontSize.setValue(controller.getFontSize());
		spDegreeFontSize.setValue(controller.getDegreeFontSize());
		spAnimationLength.setValue(controller.getAnimationLength());

		// Colors
		edgeColor = ColorLib.getColor(controller.getEdgeColor());
		strokeColor = ColorLib.getColor(controller.getStrokeColor());
	}

	private void saveChanges() {
		controller.changeLayoutParameters(chckbxEnabledFirstStages.isSelected(),
				(LayeringStrategy) cbLayeringStrategy.getSelectedItem(),
				(LayoutOrientation) cbLayoutOrientation.getSelectedItem(), chckbxEnableSimplifier.isSelected(),
				chckbxSimplifyAll.isSelected(), chckbxSimplifyLongEdges.isSelected(), chckbxEdgeDirection.isSelected(),
				chckbxBarDrawing.isSelected(), chckbxSimplifySameLayer.isSelected(), chckbxReorderNodes.isSelected(),
				(int) spCrossingThreshold.getValue(), (int) spTotalCrossings.getValue(), (int) spMaxLoops.getValue(),
				(int) spIntracellSpacing.getValue(), (int) spLayerSpacing.getValue(),
				(float) spEdgeLandingOffset.getValue(), (SequencingStrategy) cbSequencingStrategy.getSelectedItem(),
				(EmplacingStrategy) cbEmplacingStrategy.getSelectedItem(), (LineType) cbLineType.getSelectedItem(),
				(int) spTransitionsWidth.getValue(), (int) spTransitionsHeight.getValue(),
				chckbxTransitionsInOutDegree.isSelected(), (int) spPlacesWidth.getValue(),
				(int) spPlacesWidth.getValue(), (int) spFontSize.getValue(), (int) spDegreeFontSize.getValue(),
				(long) spAnimationLength.getValue(), ColorLib.color(edgeColor), ColorLib.color(strokeColor));
	}

	private void setSimplifierPropertiesEnabled(boolean enabled) {
		chckbxSimplifyAll.setEnabled(enabled);
		chckbxSimplifyLongEdges.setEnabled(enabled);
		chckbxEdgeDirection.setEnabled(enabled);
		chckbxBarDrawing.setEnabled(enabled);
		spCrossingThreshold.setEnabled(enabled);
		spTotalCrossings.setEnabled(enabled);
		spMaxLoops.setEnabled(enabled);
		chckbxSimplifySameLayer.setEnabled(enabled);
		chckbxReorderNodes.setEnabled(enabled);

		if (enabled) {
			setSimplifyShortLongPropertiesEnabled(chckbxSimplifyAll.isSelected() || chckbxSimplifyLongEdges.isSelected());
		}

		//
		//		if (!enabled) {
		//			chckbxSimplifyAll.setSelected(false);
		//			chckbxSimplifyLongEdges.setSelected(false);
		//			chckbxEdgeDirection.setSelected(false);
		//			chckbxBarDrawing.setSelected(false);
		//			chckbxSimplifySameLayer.setSelected(false);
		//			chckbxReorderNodes.setSelected(false);
		//		}
	}

	private void setSimplifyShortLongPropertiesEnabled(boolean enabled) {
		spCrossingThreshold.setEnabled(!enabled);
		spTotalCrossings.setEnabled(!enabled);
		spMaxLoops.setEnabled(!enabled);
		chckbxSimplifySameLayer.setEnabled(!enabled);
	}

	private void setFirstStagesEnabled(boolean enabled) {
		cbLayeringStrategy.setEnabled(enabled);
		cbSequencingStrategy.setEnabled(enabled);
	}

	public static void addEscapeListener(final JDialog dialog) {
		ActionListener escListener = new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				dialog.setVisible(false);
			}
		};
		dialog.getRootPane().registerKeyboardAction(escListener, KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0),
				JComponent.WHEN_IN_FOCUSED_WINDOW);
	}
}
