package org.processmining.humbertogarciacaballero.models.model.layout.stage;

import java.util.Collections;
import java.util.Iterator;

import org.processmining.humbertogarciacaballero.models.model.layout.HierarchicalLayouter;
import org.processmining.humbertogarciacaballero.models.model.layout.HierarchicalLayouter.EdgeParams;
import org.processmining.humbertogarciacaballero.models.model.layout.stage.cycle.CycleDetector;
import org.processmining.humbertogarciacaballero.models.model.layout.stage.cycle.CycleVisitor;
import org.processmining.humbertogarciacaballero.models.util.GraphUtil;

import prefuse.data.Edge;
import prefuse.data.Graph;
import prefuse.data.Node;

public class CycleRemover extends LayoutStage implements CycleVisitor {

    public CycleRemover(HierarchicalLayoutContext context) {
        super(context);
    }

    public int run(Graph graph) {
        CycleDetector.detectCycles(graph, this);
        
        return 0;
    }
    
    @Override
    public void visitCycle(Graph graph, Node source, Node target, Edge edge) {
        Edge opposite = graph.getEdge(target, source);
        Edge newEdge = GraphUtil.reverseEdge(graph, edge);
        EdgeParams params = (EdgeParams) newEdge.get(HierarchicalLayouter.EDGE_PARAMS);
        params.reversed = true;

        // This means that the new edge already existed
        if (opposite != null) {
            params.bidirectionalCycles = true;
        }
    }

    @SuppressWarnings("unchecked")
    public void redoCycles(Graph graph) {
        Iterator<Edge> iter = graph.edges();
        while (iter.hasNext()) {
            Edge e = iter.next();
            EdgeParams originalParams = (EdgeParams) e.get(HierarchicalLayouter.EDGE_PARAMS);
            if (originalParams.reversed) {
                EdgeParams clonedParams = originalParams.clone();
                Edge reversedEdge;
                if (originalParams.bidirectionalCycles) {
                    reversedEdge = graph.addEdge(e.getTargetNode(), e.getSourceNode());
                    originalParams.reversed = clonedParams.reversed = originalParams.bidirectionalCycles = clonedParams.bidirectionalCycles = false;
                    clonedParams.sourceOffset = originalParams.bidirectionalSourceOffset;
                    clonedParams.targetOffset = originalParams.bidirectionalTargetOffset;
                } else {
                    reversedEdge = GraphUtil.reverseEdge(graph, e);
                    clonedParams.reversed = clonedParams.bidirectionalCycles = false;
                }

                Collections.reverse(clonedParams.bends);
                float aux = clonedParams.sourceOffset;
                clonedParams.sourceOffset = clonedParams.targetOffset;
                clonedParams.targetOffset = aux;
                reversedEdge.set(HierarchicalLayouter.EDGE_PARAMS, clonedParams);
            }
        }
    }
}
