package org.processmining.humbertogarciacaballero.models.model.layout.stage;

import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.processmining.humbertogarciacaballero.models.model.PNConstants;
import org.processmining.humbertogarciacaballero.models.model.layout.data.LayerContainer;
import org.processmining.humbertogarciacaballero.models.model.layout.data.LayoutOrientation;

public class HierarchicalLayoutContext implements Cloneable {

    // Data structures
    private LayerContainer layerContainer;
    private List<Object> simplificationParameters;
    private Map<Long, Integer> colorMap;

    // Parameters
    private boolean firstStagesEnabled;

    private int maxIterations;

    private int maxNoImprovementIterations;

    private int crossingThreshold;

    private int simplifierMaxLoops;

    private int totalCrossingThreshold;

    private boolean simplifierActive;

    private boolean simplifyShortEdgesEnabled;

    private boolean simplifyLongEdgesEnabled;

    private boolean edgeDirectionEnabled;

    private boolean simplifySameLayer;
    
    private boolean reorderNodes;

    // Emplacer parameters
    private LayoutOrientation layoutOrientation;

    private int layerSpacing;

    private int intracellSpacing;

    private float edgeLandingOffset;

    private float portSeparation;

    private static HierarchicalLayoutContext instance = new HierarchicalLayoutContext();

    public static HierarchicalLayoutContext getInstance() {
        return instance;
    }

    private HierarchicalLayoutContext() {
        setFirstStagesEnabled(true);

        // Layerer
        setLayerContainer(new LayerContainer());

        // Sequencer
        setMaxIterations(24);
        setMaxNoImprovementIterations(4);

        // Simplifier
        setSimplifierActive(false);
        setSimplifyShortEdgesEnabled(false);
        setEdgeDirectionEnabled(false);
        setSimplifySameLayer(false);
        setSimplifierMaxLoops(24);
        setTotalCrossingThreshold(15);
        setCrossingThreshold(5);
        simplificationParameters = new LinkedList<>();

        // Color mapper
        setColorMap(new HashMap<Long, Integer>());

        // Emplacer
        setLayoutOrientation(LayoutOrientation.LEFT_TO_RIGHT);
        setLayerSpacing(PNConstants.LAYER_SPACING_DEFAULT);
        setIntracellSpacing(PNConstants.INTRACELL_SPACING_DEFAULT);
        setEdgeLandingOffset(PNConstants.EDGE_LANDING_OFFSET_DEFAULT);
        setPortSeparation(5f);
    }

    public void setLayerContainer(LayerContainer layerContainer) {
        this.layerContainer = layerContainer;
    }

    public LayerContainer getLayerContainer() {
        return layerContainer;
    }

    public int getMaxIterations() {
        return maxIterations;
    }

    public void setMaxIterations(int maxIterations) {
        this.maxIterations = maxIterations;
    }

    public int getMaxNoImprovementIterations() {
        return maxNoImprovementIterations;
    }

    public void setMaxNoImprovementIterations(int maxNoImprovementIterations) {
        this.maxNoImprovementIterations = maxNoImprovementIterations;
    }

    public int getCrossingThreshold() {
        return crossingThreshold;
    }

    public void setCrossingThreshold(int crossingThreshold) {
        this.crossingThreshold = crossingThreshold;
    }

    public Map<Long, Integer> getColorMap() {
        return colorMap;
    }

    public void setColorMap(Map<Long, Integer> colorMap) {
        this.colorMap = colorMap;
    }

    public LayoutOrientation getLayoutOrientation() {
        return layoutOrientation;
    }

    public void setLayoutOrientation(LayoutOrientation layoutOrientation) {
        this.layoutOrientation = layoutOrientation;
    }

    public int getLayerSpacing() {
        return layerSpacing;
    }

    public void setLayerSpacing(int layerSpacing) {
        this.layerSpacing = layerSpacing;
    }

    public int getIntracellSpacing() {
        return intracellSpacing;
    }

    public void setIntracellSpacing(int intracellSpacing) {
        this.intracellSpacing = intracellSpacing;
    }

    public float getEdgeLandingOffset() {
        return edgeLandingOffset;
    }

    public void setEdgeLandingOffset(float edgeLandingOffset) {
        this.edgeLandingOffset = edgeLandingOffset;
    }

    public int getSimplifierMaxLoops() {
        return simplifierMaxLoops;
    }

    public void setSimplifierMaxLoops(int simplifierMaxLoops) {
        this.simplifierMaxLoops = simplifierMaxLoops;
    }

    public int getTotalCrossingThreshold() {
        return totalCrossingThreshold;
    }

    public void setTotalCrossingThreshold(int totalCrossingThreshold) {
        this.totalCrossingThreshold = totalCrossingThreshold;
    }

    public boolean isSimplifierActive() {
        return simplifierActive;
    }

    public void setSimplifierActive(boolean simplifierActive) {
        this.simplifierActive = simplifierActive;
    }

    public void setSimplifyShortEdgesEnabled(boolean enabled) {
        simplifyShortEdgesEnabled = enabled;
    }

    public boolean isSimplifyShortEdgesEnabled() {
        return simplifyShortEdgesEnabled;
    }

    public void setEdgeDirectionEnabled(boolean enabled) {
        edgeDirectionEnabled = enabled;
    }

    public boolean isEdgeDirectionEnabled() {
        return edgeDirectionEnabled;
    }

    public void resetSimplification() {
        simplificationParameters.clear();
    }

    public void addSimplificationParameter(Object parameter) {
        simplificationParameters.add(parameter);
    }

    public Iterator<Object> simplificationParameters() {
        return simplificationParameters.iterator();
    }

    public boolean isSimplifySameLayer() {
        return simplifySameLayer;
    }

    public void setSimplifySameLayer(boolean simplifySameLayer) {
        this.simplifySameLayer = simplifySameLayer;
    }
    
    public boolean isReorderNodesEnabled() {
        return reorderNodes;
    }
    
    public void setReorderNodesEnabled(boolean enabled) {
        reorderNodes = enabled;
    }

    public float getPortSeparation() {
        return portSeparation;
    }

    public void setPortSeparation(float portSeparation) {
        this.portSeparation = portSeparation;
    }

    // public static void updateInstance(HierarchicalLayoutContext other) {
    //// instance.setColorMap(other.getColorMap());
    // instance.setCrossingThreshold(other.getCrossingThreshold());
    // instance.setEdgeDirectionEnabled(other.isEdgeDirectionEnabled());
    // instance.setEdgeLandingOffset(other.getEdgeLandingOffset());
    // instance.setIntracellSpacing(other.getIntracellSpacing());
    //// instance.setLayerContainer(other.getLayerContainer());
    // instance.setLayerSpacing(other.getLayerSpacing());
    // instance.setLayoutOrientation(other.getLayoutOrientation());
    // instance.setMaxIterations(other.getMaxIterations());
    // instance.setMaxNoImprovementIterations(other.getMaxNoImprovementIterations());
    // instance.setPortSeparation(other.getPortSeparation());
    // instance.setSimplifierActive(other.isSimplifierActive());
    // instance.setSimplifierMaxLoops(other.getSimplifierMaxLoops());
    // instance.setSimplifyAllEdgesEnabled(other.isSimplifyAllEdgesEnabled());
    // instance.setSimplifySameLayer(other.isSimplifySameLayer());
    // instance.setTotalCrossingThreshold(other.getTotalCrossingThreshold());
    // }

    // public HierarchicalLayoutContext clone() {
    // Object obj = null;
    // try {
    // obj = super.clone();
    // } catch (CloneNotSupportedException e) {
    // assert false: "The object cannot be cloned";
    // }
    // HierarchicalLayoutContext copy = (HierarchicalLayoutContext) obj;
    // copy.colorMap = new HashMap<>(this.colorMap);
    // copy.layerContainer = new LayerContainer(this.layerContainer);
    // copy.simplificationParameters = new
    // LinkedList<>(this.simplificationParameters);
    // return copy;
    // }

    public boolean isFirstStagesEnabled() {
        return firstStagesEnabled;
    }

    public void setFirstStagesEnabled(boolean enabled) {
        firstStagesEnabled = enabled;
    }

    public void setSimplifyLongEdgesEnabled(boolean enabled) {
        simplifyLongEdgesEnabled = enabled;
    }

    public boolean isSimplifyLongEdgesEnabled() {
        return simplifyLongEdgesEnabled;
    }
}
