package org.processmining.humbertogarciacaballero.models.model.layout.stage.layerer;

import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.processmining.humbertogarciacaballero.models.model.layout.stage.HierarchicalLayoutContext;

import prefuse.data.Edge;
import prefuse.data.Graph;
import prefuse.data.Node;

public class CoffmanGrahamLayerer extends Layerer {

	private int w = 3;
	
	public CoffmanGrahamLayerer(HierarchicalLayoutContext context) {
		super(context);
	}

	@SuppressWarnings("unchecked")
    @Override
	public void calculateLayers(Graph graph) {
		Iterator<Node> iter = graph.nodes();
		Map<Node, Integer> labels = new HashMap<>();
		List<Node> nonPlacedNodes = new LinkedList<>();
		
		while (iter.hasNext()) {
			Node n = iter.next();
			labels.put(n, Integer.MIN_VALUE);
			nonPlacedNodes.add(n);
		}

		iter = getRootNodes(graph);
		int i = 0;
		while (iter.hasNext()) {
			labels.put(iter.next(), i++);
		}
		
		for (; i < graph.getNodeCount(); i++) {
			// Gets next valid node
			Node selected = null;
			iter = graph.nodes();
			while (iter.hasNext() && selected == null) {
				Node v = iter.next();
				if (labels.get(v) == Integer.MIN_VALUE) {
					boolean valid = true;
					Iterator<Node> iter2 = v.inNeighbors();
					while (iter2.hasNext() && valid) {
						Node u = iter2.next();
						if (labels.get(u) == Integer.MIN_VALUE) {
							valid = false;
						}
					}
					
					if (valid) selected = v;
				}				
			}
			
			// Always find a node
			labels.put(selected, i);
		}
		
		int layer = 0;
		
		while (nonPlacedNodes.size() > 0) {
			Node candidate = null;
			int maxLayer = -1;
			for (Node u : nonPlacedNodes) {
				boolean targetsPlaced = true;
				Iterator<Node> iter2 = u.outNeighbors();
				while (iter2.hasNext() && targetsPlaced) {
					Node v = iter2.next();
					if (nonPlacedNodes.contains(v)) {
						targetsPlaced = false;
					} else {
						maxLayer = Math.max(maxLayer, getLayer(v));
					}
				}
				
				if (targetsPlaced) {
					if (candidate == null || (candidate != null && labels.get(candidate) < labels.get(u))) {
						candidate = u;
					}
				}
			}
			
			if (getContext().getLayerContainer().getSize(layer) < w) {
				setLayer(candidate, maxLayer + 1);
			} else {
				layer++;
				setLayer(candidate, layer);
			}
			
			nonPlacedNodes.remove(candidate);
		}
		
		int maxLayer = getContext().getLayerContainer().getHighestLayer();
		iter = graph.nodes();
		while (iter.hasNext()) {
			Node n = iter.next();
			moveNode(n, maxLayer - getLayer(n));
		}
		
		List<Edge> edgesDummy = new LinkedList<>();
		Iterator<Edge> iter2 = graph.edges();
		while (iter2.hasNext()) {
			Edge e = iter2.next();
			Node u = e.getSourceNode();
			Node v = e.getTargetNode();
			
			if (Math.abs(getLayer(u) - getLayer(v)) > 1) {
				edgesDummy.add(e);
			}
		}
		
		for (Edge e : edgesDummy) {
			createDummies(graph, e.getSourceNode(), e.getTargetNode());
		}
	}

}
