package org.processmining.humbertogarciacaballero.models.model.layout.stage.layerer;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Iterator;

import org.apache.commons.math3.optim.PointValuePair;
import org.apache.commons.math3.optim.linear.LinearConstraint;
import org.apache.commons.math3.optim.linear.LinearConstraintSet;
import org.apache.commons.math3.optim.linear.LinearObjectiveFunction;
import org.apache.commons.math3.optim.linear.NonNegativeConstraint;
import org.apache.commons.math3.optim.linear.Relationship;
import org.apache.commons.math3.optim.linear.SimplexSolver;
import org.apache.commons.math3.optim.nonlinear.scalar.GoalType;
import org.processmining.humbertogarciacaballero.models.model.layout.stage.HierarchicalLayoutContext;

import prefuse.data.Edge;
import prefuse.data.Graph;
import prefuse.data.Node;

public class SimplexLayerer extends Layerer {

	public SimplexLayerer(HierarchicalLayoutContext context) {
		super(context);
	}
	
	@SuppressWarnings("unchecked")
    @Override
	public void calculateLayers(Graph graph) {
		double[] coeficients = new double[graph.getNodes().getTupleCount()];
		Arrays.fill(coeficients, 0d);
		Collection<LinearConstraint> constraints = new ArrayList<LinearConstraint>();
		Iterator<Edge> iter = graph.edges();
		
		while (iter.hasNext()) {
			Edge e = iter.next();
			Node v = e.getSourceNode();
			Node w = e.getTargetNode();
			int indexV = v.getRow();
			int indexW = w.getRow();
			
			coeficients[indexW]++;
			coeficients[indexV]--;
			
			double[] constraintCoeficients = new double[coeficients.length];
			Arrays.fill(constraintCoeficients, 0);
			constraintCoeficients[indexW] = 1;
			constraintCoeficients[indexV] = -1;
			constraints.add(new LinearConstraint(constraintCoeficients, Relationship.GEQ, 1));
		}
		
		LinearObjectiveFunction f = new LinearObjectiveFunction(coeficients, 0);

		SimplexSolver solver = new SimplexSolver();
		PointValuePair pointValuePair = solver.optimize(f, new LinearConstraintSet(constraints), GoalType.MINIMIZE, new NonNegativeConstraint(true));
		
		double[] solution = pointValuePair.getPoint();
		
		for (int i = 0; i < coeficients.length; i++) {
			setLayer(graph.getNode(i), (int) solution[i]);
		}
		
		iter = graph.edges();
		while (iter.hasNext()) {
			Edge e = iter.next();
			Node v = e.getSourceNode();
			Node w = e.getTargetNode();
			
			createDummies(graph, v, w);
		}
	}

}
