package org.processmining.prabhakardixit.processanalytics;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.extension.std.XTimeExtension;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.plugins.petrinet.replayresult.PNRepResult;
import org.processmining.plugins.petrinet.replayresult.StepTypes;
import org.processmining.plugins.replayer.replayresult.SyncReplayResult;
import org.processmining.prabhakardixit.processanalytics.models.TimeView;

public class PairwisePerformanceInfo {

	private Set<PairwisePerformance> pairwisePerformance = new HashSet<PairwisePerformance>();

	public void updateTransitionPairs(Transition t1, Transition t2, Double time) {
		boolean updated = false;
		for (PairwisePerformance pp : pairwisePerformance) {
			if (pp.getTransition1().equals(Integer.toString(t1.hashCode())))
				if (pp.getTransition2().equals(Integer.toString(t2.hashCode()))) {
					updated = true;
					pp.addTime(time);
				}
		}
		if (!updated) {
			pairwisePerformance.add(new PairwisePerformance(t1, t2, time));

		}
	}

	public Double getTotalCount(Transition t1, Transition t2) {
		double sum = 0;
		for (PairwisePerformance pp : pairwisePerformance) {
			if (pp.getTransition1().equals(t1))
				if (pp.getTransition2().equals(t2)) {
					for (Double l : pp.getTime()) {
						sum += l;
					}
				}
		}
		return (double) sum;
	}

	public Double getAverageCount(Transition t1, Transition t2) {
		double sum = 0;
		int count = 0;
		for (PairwisePerformance pp : pairwisePerformance) {
			if (pp.getTransition1().equals(t1))
				if (pp.getTransition2().equals(t2)) {
					for (Double l : pp.getTime()) {
						sum += l;
						count++;
					}
				}
		}
		return (double) sum / count;
	}

	public Double getMedian(Transition t1, Transition t2) {
		List<Double> numArray = new ArrayList<Double>();
		for (PairwisePerformance pp : pairwisePerformance) {
			if (pp.getTransition1().equals(t1))
				if (pp.getTransition2().equals(t2)) {
					numArray.addAll(pp.getTime());
				}
		}
		Collections.sort(numArray);
		double median;
		if (numArray.size() % 2 == 0)
			median = ((double) numArray.get(numArray.size() / 2) + (double) numArray.get(numArray.size() / 2 - 1)) / 2;
		else
			median = (double) numArray.get(numArray.size() / 2);
		return median;
	}

	public void print(Petrinet pn) {
		for (Transition t1 : pn.getTransitions()) {
			for (Transition t2 : pn.getTransitions()) {
				if (t1 != t2) {
					System.out.println("Average : " + t1 + " : " + t2 + " : " + getAverageCount(t1, t2));
				}
			}
		}
	}

	public void clear() {
		pairwisePerformance.clear();
	}

	public void updatePerformance(PNRepResult replayResult, XLog log) {
		pairwisePerformance.clear();
		for (SyncReplayResult sync : replayResult) {

			for (int traceIndex : sync.getTraceIndex()) {
				XTrace trace = log.get(traceIndex);
				int evtIndex = 0;//increment only for log/model moves
				Transition performanceTran1 = null;
				XEvent performanceEvent1 = null;
				for (int syncIndex = 0; syncIndex < sync.getStepTypes().size(); syncIndex++) {
					StepTypes type = sync.getStepTypes().get(syncIndex);
					Object stepObj = sync.getNodeInstance().get(syncIndex);
					boolean addToCounter = true; //update only for act start specific cases
					switch (type) {
						case L :
							XEvent event = trace.get(evtIndex);
							String activity = XConceptExtension.instance().extractName(event);

							evtIndex++;
							break;
						case LMGOOD :
							event = trace.get(evtIndex);

							//performance starts
							if (performanceTran1 != null) {
								double time = ProcessAnalyticsUtils.getSelctedTimeGranularity(XTimeExtension.instance().extractTimestamp(performanceEvent1), XTimeExtension.instance().extractTimestamp(event),
										TimeView.MINUTES);
								updateTransitionPairs(performanceTran1, (Transition) stepObj, time);
							}
							performanceTran1 = (Transition) stepObj;
							performanceEvent1 = event;
							//performance ends
							evtIndex++;
							break;
						case MINVI :
						case MREAL :
							performanceTran1 = null; //reset perf indi
							performanceEvent1 = null; //reset perf indi
							break;
						case LMNOGOOD :
						case LMREPLACED :
						case LMSWAPPED :
							// unable to project this case, stop
					}

				}
			}
		}
	}
}
