package org.processmining.prabhakardixit.processanalytics;

import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import javax.swing.JPanel;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.extension.std.XTimeExtension;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.plugins.petrinet.replayresult.PNRepResult;
import org.processmining.plugins.petrinet.replayresult.StepTypes;
import org.processmining.plugins.replayer.replayresult.SyncReplayResult;
import org.processmining.prabhakardixit.featureSelection.CalculateLogHeuristics;
import org.processmining.prabhakardixit.processanalytics.models.ActivityStartOptions;
import org.processmining.prabhakardixit.processanalytics.models.ChartView;
import org.processmining.prabhakardixit.processanalytics.models.GraphData;
import org.processmining.prabhakardixit.processanalytics.models.ProcessView;
import org.processmining.prabhakardixit.processanalytics.models.TimeView;

public class ProcessAnalyticsVizViews extends ProcessAnalyticsViz {

	public ProcessAnalyticsVizViews(PNRepResult replayResult, Petrinet pn, XLog log, JPanel frame, TimeView timeView,
			ProcessView processView) {
		this.setReplayResult(replayResult); //from the plugin
		this.setReplayResult_original(replayResult);//from the plugin
		this.setPetrinet(pn); //from the plugin
		this.setLog(log); //from the plugin
		this.loadLogMovePlaces();
		this.setTimeView(timeView);
		this.setPanel(frame);
		this.processView = ProcessView.CaseStart;
		this.graphView = ChartView.STACKEDAREACHART;

	}

	public ProcessAnalyticsVizViews(PNRepResult replayResult, Petrinet pn, XLog log, TimeView hours,
			ProcessView processView, ChartView chartView, GraphData graphData) {
		this.setPetrinet(pn); //from the plugin
		this.setLog(log); //from the plugin
		this.setLogHeuristics(new CalculateLogHeuristics(log)); //create new log heuristics once
		//		this.loadLogMovePlaces();
		this.setTimeView(timeView);
		this.processView = processView;
		this.graphView = chartView;
		this.graphData = graphData;
		this.setReplayResult(replayResult); //from the plugin
		this.setReplayResult_original(replayResult);//from the plugin
	}

	public static void recomputeAlignments() { //update current replay result
		if (!tranSet.isEmpty())
			setReplayResult(RecomputeAlignments.recalculateAlignments(context, log, pn, 10, tranSet));
	}

	public static void updatePerformanceDetails() {
		pairwisePerformance.updatePerformance(replayResult_original, log);
		pairwisePerformance.print(getPetrinet());
	}

	public static void createGraph() {
		Set<Transition> tranSet2 = new HashSet<Transition>(pn.getTransitions());
		if (!tranSet.isEmpty())
			tranSet2 = tranSet;
		ProcessAnalyticsUtils.initializeCounter(synchornousCount, tranSet2);
		switch (processView) {
			case ActivityStart :
				Set<Integer> syncHashCodes = ProcessAnalyticsUtils.getAllSyncMoveCodes(selectedTran, caseLevelInfo);
				for (SyncReplayResult sync : replayResult_current) {
					if (syncHashCodes.contains(sync.hashCode())) //only consider the sync moves w.r.t.  selected activity
						getTraceSyncFits(sync, tranSet, selectedTran);
				}
				break;
			case CaseStart :
				for (SyncReplayResult sync : replayResult_current) {
					getTraceSyncFits(sync, tranSet2, null);
				}
				break;
			case Normal :
				for (SyncReplayResult sync : replayResult_current) {
					getTraceSyncFits(sync, tranSet2, null);
				}
				break;
			default :
				break;

		}

		if (graphData.equals(GraphData.SyncMovesOnly))
			generateGraphs(panel, synchornousCount, logMoveCount, graphView);
		else
			generateGraphs(panel, logMoveCount, logMoveCount, graphView);
	}

	//Options to the user: 1) consider ALL the sync moves (incl loop activities) w.r.t. the first occurrence of selected activity
	//2) consider only the first occurrence of other activity  w.r.t. the first occurrence of selected activity
	//3) consider the average of all occurrences of other activity w.r.t. the first occurrence of selected activity
	public static void viewActivityStart(Set<Transition> tranSet, Set<Place> placeSet, Transition primaryTran) {
		ProcessAnalyticsUtils.initializeCounter(synchornousCount, tranSet);
		Set<Integer> syncHashCodes = ProcessAnalyticsUtils.getAllSyncMoveCodes(selectedTran, caseLevelInfo);
		for (SyncReplayResult sync : replayResult_current) {
			if (syncHashCodes.contains(sync.hashCode())) //only consider the sync moves w.r.t.  selected activity
				getTraceSyncFits(sync, tranSet, primaryTran);
		}
	}

	private static void getTraceSyncFits(SyncReplayResult sync, Set<Transition> tranSet, Transition primaryTran) {
		for (int traceIndex : sync.getTraceIndex()) {
			XTrace trace = log.get(traceIndex);
			Date evt1 = getFirstEvent(trace, sync, primaryTran);
			int evtIndex = 0;//increment only for log/model moves
			Map<String, Double> syncActValue = new HashMap<String, Double>(); //use these maps for specific act start cases 
			Map<String, Double> logActValue = new HashMap<String, Double>(); //use these maps for specific act start cases 
			Map<String, Integer> syncActCount = new HashMap<String, Integer>(); //use these maps for specific act start cases 
			Map<String, Integer> logActCount = new HashMap<String, Integer>(); //use these maps for specific act start cases
			int flag = 0;
			Date d1 = null, d2 = null;
			for (int syncIndex = 0; syncIndex < sync.getStepTypes().size(); syncIndex++) {
				StepTypes type = sync.getStepTypes().get(syncIndex);
				Object stepObj = sync.getNodeInstance().get(syncIndex);
				boolean addToCounter = true; //update only for act start specific cases
				switch (type) {
					case L :
						XEvent event = trace.get(evtIndex);
						String activity = XConceptExtension.instance().extractName(event);
						double key = ProcessAnalyticsUtils.getSelctedTimeGranularity(evt1, XTimeExtension.instance().extractTimestamp(event), timeView);
						addToCounter = ProcessAnalyticsUtils.addToCounter(logActValue, logActCount, processView,
								activityOption, activity, key);
						if (addToCounter) {

							if (tranSet.contains(stepObj)) //if tran selected, then add to counter
							{
								if (logMoveCount.get(activity).containsKey(key))
									logMoveCount.get(activity).put(key, logMoveCount.get(activity).get(key) + 1);
								else
									logMoveCount.get(activity).put(key, 1.);
							}
						}
						evtIndex++;
						flag = Math.max(--flag, 0);
						break;
					case LMGOOD :

						event = trace.get(evtIndex);
						activity = XConceptExtension.instance().extractName(event);
						flag++;
						if (flag == 1) {
							d1 = XTimeExtension.instance().extractTimestamp(event);
						} else if (flag == 2) {
							d2 = XTimeExtension.instance().extractTimestamp(event);

							Transition t = ProcessAnalyticsViz.findTransition(activity);
							if (ProcessAnalyticsViz.syncMovesPerformance.containsKey(t)) {
								ProcessAnalyticsViz.syncMovesPerformance.put(t,
										ProcessAnalyticsViz.syncMovesPerformance.get(t)
												+ (d2.getTime() - d1.getTime()));
							} else {
								ProcessAnalyticsViz.syncMovesPerformance.put(t, (float) (d2.getTime() - d1.getTime()));
							}

							flag = 1;
							d1 = d2;
						}

						key = ProcessAnalyticsUtils.getSelctedTimeGranularity(evt1, XTimeExtension.instance().extractTimestamp(event), timeView);
						addToCounter = ProcessAnalyticsUtils.addToCounter(syncActValue, syncActCount, processView,
								activityOption, activity, key);
						if (addToCounter) {

							if (tranSet.contains(stepObj)) //if tran selected, then add to counter
							{
								if (synchornousCount.get(activity).containsKey(key))
									synchornousCount.get(activity).put(key,
											synchornousCount.get(activity).get(key) + 1);
								else
									synchornousCount.get(activity).put(key, 1.);
							}
						}
						evtIndex++;
						break;
					case MINVI :
//						event = trace.get(evtIndex);
//						activity = XConceptExtension.instance().extractName(event);
//						flag++;
//						if (flag == 1) {
//							d1 = XTimeExtension.instance().extractTimestamp(event);
//						} else if (flag == 2) {
//							d2 = XTimeExtension.instance().extractTimestamp(event);
//
//							Transition t = ProcessAnalyticsViz.findTransition(activity);
//							if (ProcessAnalyticsViz.syncMovesPerformance.containsKey(t)) {
//								ProcessAnalyticsViz.syncMovesPerformance.put(t,
//										ProcessAnalyticsViz.syncMovesPerformance.get(t)
//												+ (d2.getTime() - d1.getTime()));
//							} else {
//								ProcessAnalyticsViz.syncMovesPerformance.put(t, (float) (d2.getTime() - d1.getTime()));
//							}
//
//							flag = 1;
//							d1 = d2;
//						}
						flag = Math.max(--flag, 0);
						break;
					case MREAL :
						flag = Math.max(--flag, 0);
						break;
					case LMNOGOOD :
						flag = Math.max(--flag, 0);
						break;
					case LMREPLACED :
						flag = Math.max(--flag, 0);
						break;
					case LMSWAPPED :
						flag = Math.max(--flag, 0);
						break;
					// unable to project this case, stop
				}
				if (syncIndex == sync.getStepTypes().size()) //check if the last type- update the sync and log move counter for special cases (eg averages etc)
					updateSpecialCases(syncActValue, logActValue, syncActCount, logActCount);

			}
		}
	}

	private static void updateSpecialCases(Map<String, Double> syncActValue, Map<String, Double> logActValue,
			Map<String, Integer> syncActCount, Map<String, Integer> logActCount) {
		if (processView.equals(ProcessView.ActivityStart) && activityOption.equals(ActivityStartOptions.AvgSyncMove)) {
			for (String act : syncActValue.keySet()) {
				double value = (syncActValue.get(act)) / (syncActCount.get(act));
				if (synchornousCount.containsKey(act)) {
					if (synchornousCount.get(act).containsKey(value)) {
						synchornousCount.get(act).put(value, synchornousCount.get(act).get(value) + 1.);
					} else {
						synchornousCount.get(act).put(value, 1.);
					}
				} else {
					Map<Double, Double> map = new HashMap<Double, Double>();
					map.put(value, 1.);
					synchornousCount.put(act, map);
				}
			}
			for (String act : logActValue.keySet()) {
				double value = (logActValue.get(act)) / (logActCount.get(act));
				if (logMoveCount.containsKey(act)) {
					if (logMoveCount.get(act).containsKey(value)) {
						logMoveCount.get(act).put(value, logMoveCount.get(act).get(value) + 1.);
					} else {
						logMoveCount.get(act).put(value, 1.);
					}
				} else {
					Map<Double, Double> map = new HashMap<Double, Double>();
					map.put(value, 1.);
					logMoveCount.put(act, map);
				}
			}
		}

	}

	private static Date getFirstEvent(XTrace trace, SyncReplayResult sync, Transition primaryTran) {

		switch (processView) {
			case ActivityStart :
				int evtIndex = 0;//increment only for log/model moves
				for (int syncIndex = 0; syncIndex < sync.getStepTypes().size(); syncIndex++) {
					StepTypes type = sync.getStepTypes().get(syncIndex);
					Object stepObj = sync.getNodeInstance().get(syncIndex);
					switch (type) {
						case L :
							evtIndex++;
							break;
						case LMGOOD :
							XEvent event = trace.get(evtIndex);
							if (stepObj.equals(primaryTran))
								return XTimeExtension.instance().extractTimestamp(event);
							evtIndex++;
							break;
						default :
							break;
					}
				}
				break;
			case CaseStart :
				return XTimeExtension.instance().extractTimestamp(trace.get(0));
			case Normal :
				return logHeuristics.info.getLogTimeBoundaries().getStartDate();
			default :
				break;

		}
		return null;
	}

	private static void generateGraphs(JPanel frame, Map<String, Map<Double, Double>> displayView,
			Map<String, Map<Double, Double>> logMoveView2, ChartView graphView) {
		switch (graphView) {
			case HISTOGRAM :
				SwingGraph2.createHistogram(frame, displayView, timeView);
				break;
			case JFREESTACK :
				SwingGraph2.createStackedXY(frame, displayView, timeView); 
				break;
			case STACKEDAREACHART :
				//				StackedChart.createStackedChart2(frame,displayView);
				break;
			default :
				break;

		}
	}

	public void classify() {

		featureSelection.scoreFeatures(tranSet, this.getReplayResult(), log, panel, this.getLogHeuristics());
	}
}
