package org.processmining.prabhakardixit.processanalytics;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import nl.tue.astar.AStarException;

import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.classification.XEventNameClassifier;
import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.info.XLogInfoFactory;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.plugins.connectionfactories.logpetrinet.TransEvClassMapping;
import org.processmining.plugins.petrinet.replayer.PNLogReplayer;
import org.processmining.plugins.petrinet.replayer.algorithms.IPNReplayAlgorithm;
import org.processmining.plugins.petrinet.replayer.algorithms.costbasedcomplete.CostBasedCompleteParam;
import org.processmining.plugins.petrinet.replayer.annotations.PNReplayAlgorithm;
import org.processmining.plugins.petrinet.replayresult.PNRepResult;

public class RecomputeAlignments {
	public static PNRepResult recalculateAlignments(PluginContext context, XLog log, Petrinet ptnet,
			int maxTraceLength, Set<Transition> tranSets) {
		boolean verbose=false;
		boolean selectedActivities = false; //if no activities are selected then consider the entire net for analysis
		if (tranSets != null)
			selectedActivities = true;

//		Petrinet ptnet = apn.getPetriNet();
		TransEvClassMapping oldMap = transitionToEventClassMapperByLabel(log, ptnet);
		XLogInfo logInfo = XLogInfoFactory.createLogInfo(log);
		Set<XEventClass> evtClasses = new HashSet<XEventClass>();
		for (XEventClass evt : logInfo.getEventClasses().getClasses()) {
			XEventClass evt1 = new XEventClass(evt.getId().replace("+complete", ""), evt.getIndex());
			evtClasses.add(evt1);
		}

		CostBasedCompleteParam parameter = new CostBasedCompleteParam(evtClasses, DUMMY, ptnet.getTransitions(),
				Integer.MAX_VALUE, 1);

		logInfo.getTransitionClasses().getClasses().iterator();
		Set<String> trans = new HashSet<String>();
		for (Transition t : ptnet.getTransitions()) {
			trans.add(t.toString());
			if (!t.isInvisible() && ((selectedActivities && tranSets.contains(t)) || !selectedActivities)) {
				parameter.getMapTrans2Cost().put(t, maxTraceLength);
			} else
				parameter.getMapTrans2Cost().put(t, 0);
		}

		for (XEventClass t : evtClasses) {
			parameter.getMapEvClass2Cost().remove(t);
			if (trans.contains(t.getId())) {
				parameter.getMapEvClass2Cost().put(t, 5);
			} else
				parameter.getMapEvClass2Cost().put(t, 0);
		}

		parameter.setGUIMode(false);
		parameter.setCreateConn(false);

		Marking initialMarking = new Marking();
		initialMarking.add(ProcessAnalyticsUtils.getInitialPlace(ptnet));
		Marking finalMarking = new Marking();
		finalMarking.add(ProcessAnalyticsUtils.getFinalPlace(ptnet));

		parameter.setMaxNumOfStates(2000);
		if (verbose) {
			System.out.println("log: ");
			for (XTrace trace : log) {
				for (XEvent event : trace) {
					System.out.print(event.getAttributes().get("concept:name") + ", ");
				}
				System.out.println();
			}

			System.out.println();
			//			System.out.println("mapping:                        "+oldMap);
			System.out.println("parameter.getMapTrans2Cost():   " + parameter.getMapTrans2Cost());
			System.out.println("parameter.getMapEvClass2Cost(): " + parameter.getMapEvClass2Cost());
		}
		Object[] parameters = new Object[] { parameter.getMapTrans2Cost(), parameter.getMaxNumOfStates(),
				parameter.getMapEvClass2Cost(), 1 };

		// select algorithm without ILP
		//		PNLogMatchInstancesReplayer replayer = new PNLogMatchInstancesReplayer();
		//		NBestOptAlignmentsGraphSamplingAlg alg = new NBestOptAlignmentsGraphSamplingAlg();

		PNLogReplayer replayer2 = new PNLogReplayer();

		//		PNMatchInstancesRepResult pnRepResult = null;
		PNRepResult result = null;
		try {
			IPNReplayAlgorithm[] availAlgorithms = getAvailabelAlgorithms(context, ptnet, log, oldMap);
			parameter.setFinalMarkings(finalMarking);
			parameter.setInitialMarking(initialMarking);
			result = replayer2.replayLog(context, ptnet, log, oldMap, availAlgorithms[2], parameter);
			//			pnRepResult = replayer
			//					.replayLog(context, ptnet, log, oldMap, initialMarking, finalMarking, alg, parameters);
			//
			//			Iterator<AllSyncReplayResult> allResults = pnRepResult.iterator();
			//			while (allResults.hasNext()) {
			//				AllSyncReplayResult traceResult = allResults.next();
			//
			//				System.out.println(traceResult.getCaseIDs());
			//				System.out.println(traceResult.toString());
			//
			//			}

			//			for (SyncReplayResult sync : result) {
			//				System.out.println(sync.getInfo().get(PNRepResult.TRACEFITNESS));
			//			}
		} catch (AStarException e) {
			e.printStackTrace();
		} catch (Throwable e) {
			e.printStackTrace();
		}

		return result;
	}

	public static TransEvClassMapping transitionToEventClassMapperByLabel(final XLog log, final Petrinet net) {
		TransEvClassMapping map = new TransEvClassMapping(new XEventNameClassifier(), DUMMY);

		List<Transition> listTrans = new ArrayList<Transition>(net.getTransitions());
		Map<String, XEventClass> classes = new HashMap<String, XEventClass>();
		int index = 0;
		for (Transition transition : listTrans) {
			if (transition.isInvisible()) {
				map.put(transition, DUMMY);
			} else {
				XEventClass eClass = null;
				if (classes.containsKey(transition.getLabel()))
					eClass = classes.get(transition.getLabel());
				else {
					eClass = new XEventClass(transition.getLabel(), index);
					classes.put(transition.getLabel(), eClass);
					index++;
				}
				map.put(transition, eClass);
			}
		}
		return map;
	}

	// dummy event class (for unmapped transitions)
	public static final XEventClass DUMMY = new XEventClass("DUMMY", -1) {
		public boolean equals(Object o) {
			return this == o;
		}

		public int hashCode() {
			return System.identityHashCode(this);
		}
	};

	protected static IPNReplayAlgorithm[] getAvailabelAlgorithms(PluginContext context, final PetrinetGraph net,
			final XLog log, final TransEvClassMapping mapping) {
		// get all algorithms from the framework
		Set<Class<?>> coverageEstimatorClasses = context.getPluginManager().getKnownClassesAnnotatedWith(
				PNReplayAlgorithm.class);
		IPNReplayAlgorithm[] availAlgorithms = null;
		if (coverageEstimatorClasses != null) {
			List<IPNReplayAlgorithm> algList = new LinkedList<IPNReplayAlgorithm>();
			for (Class<?> coverClass : coverageEstimatorClasses) {
				try {
					IPNReplayAlgorithm alg = (IPNReplayAlgorithm) coverClass.newInstance();
					if (alg.isReqWOParameterSatisfied(context, net, log, mapping)) {
						algList.add(alg);
					}
				} catch (InstantiationException e1) {
					// do nothing
				} catch (IllegalAccessException e1) {
					// do nothing
				} catch (Exception exc) {
					// do nothing
				}
			}
			Collections.sort(algList, new Comparator<IPNReplayAlgorithm>() {

				public int compare(IPNReplayAlgorithm o1, IPNReplayAlgorithm o2) {
					return o1.toString().compareTo(o2.toString());
				}
			});
			availAlgorithms = algList.toArray(new IPNReplayAlgorithm[algList.size()]);

		}
		return availAlgorithms;
	}

}
